from peek_plugin_branch._private.server.controller.MainController import (
    MainController,
)
from peek_plugin_branch.server.BranchApiABC import BranchApiABC


class BranchApi(BranchApiABC):
    def __init__(self, mainController: MainController):
        self._mainController = mainController

    # def doSomethingGood(self, somethingsDescription: str) -> DoSomethingTuple:
    #     """ Do Something Good
    #
    #     Add a new task to the users device.
    #
    #     :param somethingsDescription: An arbitrary string
    #
    #     """
    #
    #     # Here we could pass on the request to the self._mainController if we wanted.
    #     # EG self._mainController.somethingCalled(somethingsDescription)
    #
    #     return DoSomethingTuple(result="SUCCESS : " + somethingsDescription)

    def shutdown(self):
        pass



import logging
from typing import Union

from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_plugin_branch._private.storage.BranchDetailTable import (
    BranchDetailTable,
)

logger = logging.getLogger(__name__)


class BranchDetailTupleProvider(TuplesProviderABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        modelSetKey = tupleSelector.selector["modelSetKey"]

        session = self._ormSessionCreator()
        try:
            ormItems = (
                session.query(BranchDetailTable)
                .filter(BranchDetailTable.modelSetKey == modelSetKey)
                .all()
            )

            items = [o.toTuple() for o in ormItems]

            # Create the vortex message
            return (
                Payload(filt, tuples=items).makePayloadEnvelope().toVortexMsg()
            )

        finally:
            session.close()



from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_plugin_branch._private.PluginNames import branchFilt
from peek_plugin_branch._private.PluginNames import branchObservableName
from vortex.handler.TupleDataObservableProxyHandler import (
    TupleDataObservableProxyHandler,
)


def makeDeviceTupleDataObservableProxy():
    return TupleDataObservableProxyHandler(
        observableName=branchObservableName,
        proxyToVortexName=peekServerName,
        additionalFilt=branchFilt,
    )



"""Removed location field

Peek Plugin Database Migration Script

Revision ID: 87f770b6a183
Revises: 53e411bea4ab
Create Date: 2019-05-17 09:33:15.251151

"""

# revision identifiers, used by Alembic.
revision = "87f770b6a183"
down_revision = "53e411bea4ab"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("BranchDetail", "location", schema="pl_branch")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "BranchDetail",
        sa.Column(
            "location", sa.INTEGER(), autoincrement=False, nullable=False
        ),
        schema="pl_branch",
    )
    # ### end Alembic commands ###



from vortex.handler.TupleActionProcessor import TupleActionProcessor

from peek_plugin_branch._private.PluginNames import branchFilt
from peek_plugin_branch._private.PluginNames import branchActionProcessorName
from .controller.MainController import MainController


def makeTupleActionProcessorHandler(mainController: MainController):
    processor = TupleActionProcessor(
        tupleActionProcessorName=branchActionProcessorName,
        additionalFilt=branchFilt,
        defaultDelegate=mainController,
    )
    return processor



import logging

from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.TupleAction import TupleActionABC
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleActionProcessor import TupleActionProcessorDelegateABC
from vortex.handler.TupleDataObservableHandler import TupleDataObservableHandler

from peek_plugin_branch._private.storage.BranchDetailTable import (
    BranchDetailTable,
)
from peek_plugin_branch._private.tuples.CreateBranchActionTuple import (
    CreateBranchActionTuple,
)
from peek_plugin_branch.tuples.BranchDetailTuple import BranchDetailTuple

logger = logging.getLogger(__name__)


class MainController(TupleActionProcessorDelegateABC):
    def __init__(
        self, dbSessionCreator, tupleObservable: TupleDataObservableHandler
    ):
        self._dbSessionCreator = dbSessionCreator
        self._tupleObservable = tupleObservable

    def shutdown(self):
        pass

    def processTupleAction(self, tupleAction: TupleActionABC) -> Deferred:
        if isinstance(tupleAction, CreateBranchActionTuple):
            return self._processCreateBranch(tupleAction)

        raise NotImplementedError(tupleAction.tupleName())

    @deferToThreadWrapWithLogger(logger)
    def _processCreateBranch(self, action: CreateBranchActionTuple):
        try:
            # Perform update using SQLALchemy
            dbSession = self._dbSessionCreator()

            dbItem = None
            mergeItem = BranchDetailTable.fromTuple(action.branchDetail)

            if action.branchDetail.id is not None:
                rows = (
                    dbSession.query(BranchDetailTable)
                    .filter(BranchDetailTable.id == action.branchDetailId)
                    .all()
                )
                if rows:
                    dbItem = rows[0]
                    dbItem.merge(mergeItem)

            if not dbItem:
                newItem = BranchDetailTable.fromTuple(action.branchDetail)
                dbSession.add(newItem)

            dbSession.commit()

            # Notify the observer of the update
            # This tuple selector must exactly match what the UI observes
            tupleSelector = TupleSelector(
                BranchDetailTuple.tupleName(),
                dict(modelSetKey=newItem.modelSetKey),
            )
            self._tupleObservable.notifyOfTupleUpdate(tupleSelector)

        except Exception as e:
            logger.exception(e)

        finally:
            # Always close the dbSession after we create it
            dbSession.close()
